==> demoniac-0.3.tar.gz: demoniac-0.3/libfx/optrect.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * 3D Position option for calodox demoniac
 * leto@calodox.org, 2001
 */

#include "opengl.h"
#include <fx.h>
#include "fxinstance.h"
#include "genericoption.h"
#include "optrect.h"
#include "interpolator.h"
#include "mousecontroller.h"
#include "glmain.h"

#define CLICK_DISTANCE 0.01

void registerRectangleOpt(char *optName, int flags, float *values){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptRectangle(currentFX, optName, flags, values) );
}

OptRectangle::OptRectangle(FXInstance *fxi, char *optName, int flags, float *values) :
		GenericOption(fxi, optName, "Screen Position", flags), 
		fvalues(values),
		grabbedCorner(NONE)
{

	values[TOP] = values[LEFT] = 0.2;
	values[BOTTOM] = values[RIGHT] = 0.8;

	this->value.set(values, 4);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
	}
}

OptRectangle::~OptRectangle() {
	if (GTK_TOGGLE_BUTTON (toggle)->active)
		glMainptr->mc->clear();
}

void OptRectangle::selected(bool state) {
	if (!state) 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), false);
	glMainptr->reDraw();
}

void OptRectangle::clicked(int button, double x, double y, int width, int height) {
	float click[2];

	if (button == 0) {
		click[0] = (float) (x/ (double) width);
		click[1] = (float) (y/ (double) height);
	}

	double dleft = fabs(click[0] - fvalues[LEFT]);
	double dright = fabs(click[0] - fvalues[RIGHT]);
	double dtop = fabs(click[1] - fvalues[TOP]);
	double dbottom = fabs(click[1] - fvalues[BOTTOM]);

	if ( dleft < dright ) {
		if (dtop < dbottom)
			grabbedCorner = TOPLEFT;
		else
			grabbedCorner = BOTTOMLEFT;
	} else {
		if (dtop < dbottom)
			grabbedCorner = TOPRIGHT;
		else
			grabbedCorner = BOTTOMRIGHT;
	}
	dragged(button, x,y,x,y, width, height);
}

void OptRectangle::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {

	float click[2];

	click[0] = (float) (xEnd/ (double) width);
	click[1] = (float) (yEnd/ (double) height);

	if (click[0] > fvalues[RIGHT]) {
		if (grabbedCorner == TOPLEFT)
			grabbedCorner = TOPRIGHT;
		if (grabbedCorner == BOTTOMLEFT)
			grabbedCorner = BOTTOMRIGHT;
	} else {
		if (click[0] < fvalues[LEFT]) {
			if (grabbedCorner == TOPRIGHT)
				grabbedCorner = TOPLEFT;
			if (grabbedCorner == BOTTOMRIGHT)
				grabbedCorner = BOTTOMLEFT;
		} 
	}

	if (click[1] > fvalues[BOTTOM]) {
		if (grabbedCorner == TOPLEFT)
			grabbedCorner = BOTTOMLEFT;
		if (grabbedCorner == TOPRIGHT)
			grabbedCorner = BOTTOMRIGHT;
	} else {
		if (click[1] < fvalues[TOP]) {
			if (grabbedCorner == BOTTOMLEFT)
				grabbedCorner = TOPLEFT;
			if (grabbedCorner == BOTTOMRIGHT)
				grabbedCorner = TOPRIGHT;
		}
	}


	

	fvalues[grabbedCorner == TOPLEFT || grabbedCorner == BOTTOMLEFT ? LEFT : RIGHT] = click[0];
	fvalues[grabbedCorner == TOPLEFT || grabbedCorner == TOPRIGHT ? TOP : BOTTOM] = click[1];

	value.set(fvalues, 4);

	if (track) {
		track->setValue(glMainptr->getTimeMilliSec(),
				&value);
	}
	glMainptr->reDraw();
}

void OptRectangle::draw() {

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDisable(GL_DEPTH_TEST);
	glLineStipple(1,0xF00F);
	glEnable(GL_LINE_STIPPLE);
	glDisable(GL_BLEND);
	
	glBegin(GL_LINE_STRIP);
	glColor4f(1,1,1,1);
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glEnd();

	glLineStipple(1,0x0FF0);
	glBegin(GL_LINE_STRIP);
	glColor4f(0,0,0,1);
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glEnd();
	glDisable(GL_LINE_STIPPLE);

}

void OptRectangle::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		value.getFloat(fvalues, 4);
	}
}

void OptRectangle::draw(GtkBox *box) {

	ignore = false; 

	if ((flags & OPTION_REINIT) == 0) {
		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

		GtkWidget *t = gtk_toggle_button_new_with_label(name);
		toggle = GTK_TOGGLE_BUTTON(t);
		gtk_box_pack_start(GTK_BOX(hbox), t, TRUE, TRUE, 0);

		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
				GTK_SIGNAL_FUNC(selectMouseListener), (MouseListener *)this);

		drawInterp(GTK_BOX(hbox));
	}
} 

void OptRectangle::update() {
}

void OptRectangle::updateGui() {
}

void OptRectangle::save(Saver *s) {
    s->saveObjectBegin("OptRectangle");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    s->saveFloatVect("fvalues", 4, fvalues);

    if ((flags & OPTION_REINIT) == 0) {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptRectangle::load(Loader *l) {
	l->checkName("OptRectangle");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127, "name");
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("fvalues", l->nextName())==0) {
		track = 0;
		l->getFloatVect(fvalues);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}

==> demoniac-0.4.tar.gz: demoniac-0.4/libfx/optrect.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * 3D Position option for calodox demoniac
 * leto@calodox.org, 2001
 */

#include "opengl.h"
#include <fx.h>
#include "fxinstance.h"
#include "genericoption.h"
#include "optrect.h"
#include "interpolator.h"
#include "mousecontroller.h"
#include "glmain.h"

#define CLICK_DISTANCE 0.01

void registerRectangleOpt(char *optName, int flags, float *values){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptRectangle(currentFX, optName, flags, values) );
}

OptRectangle::OptRectangle(FXInstance *fxi, char *optName, int flags, float *values) :
		GenericOption(fxi, optName, "Screen Position", flags), 
		fvalues(values),
		grabbedCorner(NONE)
{

	values[TOP] = values[LEFT] = 0.2;
	values[BOTTOM] = values[RIGHT] = 0.8;

	this->value.set(values, 4);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
	}
}

OptRectangle::~OptRectangle() {
	if (GTK_TOGGLE_BUTTON (toggle)->active)
		glMainptr->mc->clear();
}

void OptRectangle::selected(bool state) {
	if (!state) 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), false);
	glMainptr->reDraw();
}

void OptRectangle::clicked(int button, double x, double y, int width, int height) {
	float click[2];

	if (button == 0) {
		click[0] = (float) (x/ (double) width);
		click[1] = (float) (y/ (double) height);
	}

	double dleft = fabs(click[0] - fvalues[LEFT]);
	double dright = fabs(click[0] - fvalues[RIGHT]);
	double dtop = fabs(click[1] - fvalues[TOP]);
	double dbottom = fabs(click[1] - fvalues[BOTTOM]);

	if ( dleft < dright ) {
		if (dtop < dbottom)
			grabbedCorner = TOPLEFT;
		else
			grabbedCorner = BOTTOMLEFT;
	} else {
		if (dtop < dbottom)
			grabbedCorner = TOPRIGHT;
		else
			grabbedCorner = BOTTOMRIGHT;
	}
	dragged(button, x,y,x,y, width, height);
}

void OptRectangle::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {

	float click[2];

	click[0] = (float) (xEnd/ (double) width);
	click[1] = (float) (yEnd/ (double) height);

	if (click[0] > fvalues[RIGHT]) {
		if (grabbedCorner == TOPLEFT)
			grabbedCorner = TOPRIGHT;
		if (grabbedCorner == BOTTOMLEFT)
			grabbedCorner = BOTTOMRIGHT;
	} else {
		if (click[0] < fvalues[LEFT]) {
			if (grabbedCorner == TOPRIGHT)
				grabbedCorner = TOPLEFT;
			if (grabbedCorner == BOTTOMRIGHT)
				grabbedCorner = BOTTOMLEFT;
		} 
	}

	if (click[1] > fvalues[BOTTOM]) {
		if (grabbedCorner == TOPLEFT)
			grabbedCorner = BOTTOMLEFT;
		if (grabbedCorner == TOPRIGHT)
			grabbedCorner = BOTTOMRIGHT;
	} else {
		if (click[1] < fvalues[TOP]) {
			if (grabbedCorner == BOTTOMLEFT)
				grabbedCorner = TOPLEFT;
			if (grabbedCorner == BOTTOMRIGHT)
				grabbedCorner = TOPRIGHT;
		}
	}


	

	fvalues[grabbedCorner == TOPLEFT || grabbedCorner == BOTTOMLEFT ? LEFT : RIGHT] = click[0];
	fvalues[grabbedCorner == TOPLEFT || grabbedCorner == TOPRIGHT ? TOP : BOTTOM] = click[1];

	value.set(fvalues, 4);

	if (track) {
		track->setValue(owner->absToRel(glMainptr->getTimeMilliSec()),
				&value);
	}
	glMainptr->reDraw();
}

void OptRectangle::draw() {

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDisable(GL_DEPTH_TEST);
	glLineStipple(1,0xF00F);
	glEnable(GL_LINE_STIPPLE);
	glDisable(GL_BLEND);
	
	glBegin(GL_LINE_STRIP);
	glColor4f(1,1,1,1);
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glEnd();

	glLineStipple(1,0x0FF0);
	glBegin(GL_LINE_STRIP);
	glColor4f(0,0,0,1);
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[TOP]*2 );
	glVertex2f( fvalues[RIGHT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[BOTTOM]*2 );
	glVertex2f( fvalues[LEFT]*2-1, 1-fvalues[TOP]*2 );
	glEnd();
	glDisable(GL_LINE_STIPPLE);

}

void OptRectangle::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		value.getFloat(fvalues, 4);
	}
}

void OptRectangle::draw(GtkBox *box) {

	ignore = false; 

	if ((flags & OPTION_REINIT) == 0) {
		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

		GtkWidget *t = gtk_toggle_button_new_with_label(name);
		toggle = GTK_TOGGLE_BUTTON(t);
		gtk_box_pack_start(GTK_BOX(hbox), t, TRUE, TRUE, 0);

		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
				GTK_SIGNAL_FUNC(selectMouseListener), (MouseListener *)this);

	}
} 

void OptRectangle::update() {
}

void OptRectangle::updateGui() {
}

void OptRectangle::save(Saver *s) {
    s->saveObjectBegin("OptRectangle");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    s->saveFloatVect("fvalues", 4, fvalues);

    if ((flags & OPTION_REINIT) == 0) {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptRectangle::load(Loader *l) {
	l->checkName("OptRectangle");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127, "name");
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("fvalues", l->nextName())==0) {
		track = 0;
		l->getFloatVect(fvalues);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}
